unit LpCmbBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DB, DSGNINTF, stdctrls, IDGconst, About;

type

  TFieldBoxProperty = class (TPropertyEditor)
    function GetAttributes : TPropertyAttributes ; override;
    function GetValue: string; override;
    procedure SetValue(const Value: string);override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TFieldParamsProperty = class(TClassProperty)
  public
    function GetAttributes : TPropertyAttributes ; override;
    procedure Edit;override;
    procedure SetValue(const Value: string);override;
  end;

  TIDGDBComboBox = class(TComboBox)
  private
    FNameField : string;
    FDataLink : TDataLink;
    FFieldList : TList;
    FFieldParams : TFieldParams;

    function GetDataField :string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value : string);
    procedure SetDataSource(Value : TDataSource);

  protected
    procedure CreateWnd;override;

  public
    constructor Create(AOwner : TComponent);override;
    destructor Destroy;override;

    procedure Clear;
    procedure FillList;virtual;
    function CurrentData( FieldName : string ) : TItem;
    function ItemData( Index : Integer; FieldName : string) : TItem;
    
    function FindByName( FieldName : string; Value : Variant ) : Integer;

    function Locate( FieldNames : string; Value : Variant; Options : TLocateOptions ) : Integer;

  published
    property FieldsList : TFieldParams read FFieldParams write FFieldParams;
    property FieldName : string read GetDataField write SetDataField;
    property DataSource : TDataSource read GetDataSource write SetDataSource;
  end;

procedure Register;

implementation

uses FldFrm;


//*********************** TFieldParamsProperty ********************************//

function TFieldParamsProperty.GetAttributes : TPropertyAttributes ;
begin
   Result := [paMultiSelect,paDialog, paAutoUpdate];
end;

procedure TFieldParamsProperty.Edit;
var
  dialog : TfmFieldsEdit;
  i : Integer;
  ComboBox : TIDGDBComboBox;
  DataSet : TDataSet;
begin

  ComboBox := TIDGDBComboBox( GetComponent(0));
  DataSet := ComboBox.DataSource.DataSet;
  if ( Assigned( DataSet ) ) then begin
    dialog := TfmFieldsEdit.CreateForm(Application, DataSet );

    for i := 0 to ComboBox.FFieldParams.Count - 1 do
      dialog.AddStoredField( ComboBox.FFieldParams.Items[i].FieldName );
    dialog.FillList;

    try
      if Dialog.ShowModal = IDOK then begin
        ComboBox.FFieldParams.Clear;
      for i := 0 to dialog.lbStoredField.Items.Count - 1 do begin
        comboBox.FFieldParams.CreateParam(dialog.lbStoredField.Items[i]);
      end;
        SetValue('New');
      end;
    finally
      Dialog.Free;
    end;
  end;
end;

procedure TFieldParamsProperty.SetValue(const Value: string);
begin
  SetStrValue(Value);
end;

//************************ TFieldBoxProperty ***************************************//

function TFieldBoxProperty.GetAttributes : TPropertyAttributes ;
begin
   Result := [paMultiSelect,paValueList];
end;

procedure TFieldBoxProperty.GetValues(Proc: TGetStrProc);
var
   I : Integer;
   DSource : TDataSource;
   FActive : Boolean;
begin
  DSource := TIDGDBComboBox( GetComponent(0) ).DataSource ;
  if ( Assigned( DSource ) ) then
    if ( Assigned( DSource.DataSet ) ) then begin
      FActive := DSource.DataSet.Active;
      if not FActive then DSource.DataSet.Active := True;
      for I := 0 to DSource.DataSet.FieldCount - 1 do
        Proc( DSource.DataSet.Fields[I].FieldName );
      DSource.DataSet.Active := FActive;
    end;
end;

function TFieldBoxProperty.GetValue: string;
begin
  Result := GetStrValue;
end;

procedure TFieldBoxProperty.SetValue(const Value: string);
begin
  SetStrValue(Value);
end;

//************************* TIDGDBComboBox ***************************************//

function TIDGDBComboBox.GetDataField :string;
begin
  Result := FNameField;
end;

function TIDGDBComboBox.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TIDGDBComboBox.SetDataField(const Value : string);
begin
  FNameField := Value;
end;

procedure TIDGDBComboBox.SetDataSource(Value : TDataSource);
begin
  FDataLink.DataSource := Value;
end;


constructor TIDGDBComboBox.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FFieldParams := TFieldParams.Create;
  FDataLink := TDataLink.Create;
  FFieldList := TList.Create;
  Style := csDropDownList;
end;

destructor TIDGDBComboBox.Destroy;
begin
  FFieldParams.Free;
  FDataLink.Free;
  FFieldList.Free;
  inherited Destroy;
end;

procedure TIDGDBComboBox.Clear;
var
  i : Integer;
  id : TItemRec;
begin
  for i := 0 to FFieldList.Count - 1 do begin
    id := TItemRec( FFieldList[0]);
    id.Free;
    FFieldList.Delete(0);
  end;
  Items.Clear;
end;

procedure TIDGDBComboBox.FillList;
var
  FActive : boolean;
  Bookmark : TBookmark;
  id : TItemRec;
  i : integer;
begin
  Clear;
  if ( Assigned( DataSource ) ) then
    if ( Assigned( DataSource.DataSet ) ) then begin
      FActive := DataSource.DataSet.Active;
      if not FActive then DataSource.DataSet.Active := True;
      if FActive then
        Bookmark := DataSource.DataSet.GetBookmark;

      DataSource.DataSet.First;
      while not DataSource.DataSet.EOF do begin
        id := TItemRec.Create;
        for i := 0 to FFieldParams.Count - 1 do begin
          id.AddParam( FFieldParams.Items[i].FieldName,
                DataSource.DataSet.FieldByName(FFieldParams.Items[i].FieldName).Value );
        end;
        FFieldList.Add(id);
        Items.Add(DataSource.DataSet.FieldByName(FNameField).DisplayText);
        DataSource.DataSet.Next;
      end;
      if FActive then
        DataSource.DataSet.GotoBookmark(bookmark);
      DataSource.DataSet.Active := FActive;
    end;
  if (Items.Count > 0) then
    ItemIndex := 0;
end;


function TIDGDBComboBox.CurrentData( FieldName : string ) : TItem;
var
  id : TItemRec;
begin
  Result := nil;
  if (Items.Count > 0) and (ItemIndex > -1) then begin
    id := TItemRec(FFieldList[ItemIndex]);
    Result := id.FieldByName(FieldName);
  end
  else
    raise Exception.Create('  ');
end;

function TIDGDBComboBox.FindByName( FieldName : string; Value : Variant ) : Integer;
var
  i : Integer;
  id : TItemRec;
begin
  Result := -1;
  for i := 0 to FFieldList.Count - 1 do begin
    id := TItemRec(FFieldList[i]);
    if ( Value = id.FieldByName(FieldName).Value ) then begin
      Result := i;
      Exit;
    end;
  end;
end;

function TIDGDBComboBox.Locate( FieldNames : string; Value : Variant; Options : TLocateOptions ) : Integer;
var
  Fields : TItemRec;
  i : Integer;
  id : TItemRec;
begin
  Result := -1;
  Fields := TItemRec.Create;
  try
    GetFieldNames( Fields, FieldNames, Value );
    for i := 0 to FFieldList.Count - 1 do begin
      id := FFieldList[i];
      if CompareItemRec( Fields, id, Options ) then begin
        Result := i;
        exit;
      end;
    end;
  finally
    Fields.Free;
  end;
end;

procedure TIDGDBComboBox.CreateWnd;
begin
  inherited CreateWnd;
  if not(csDesigning in ComponentState) then begin
    FillList;
  end;
end;

function TIDGDBComboBox.ItemData(Index: Integer; FieldName: string): TItem;
var
  id : TItemRec;
begin
  if (Items.Count > Index ) and ( Index > -1) then begin
    id := TItemRec(FFieldList[Index]);
    Result := id.FieldByName(FieldName);
  end
  else
    raise Exception.Create('  ');
end;


procedure Register;
begin
  RegisterComponents('IDG', [TIDGDBComboBox]);
  RegisterComponentEditor( TIDGDBComboBox, TAboutEditor );
  RegisterPropertyEditor(TypeInfo(TFieldParams),TIDGDBComboBox,'FieldsList',TFieldParamsProperty);
  RegisterPropertyEditor(TypeInfo(string),TIDGDBComboBox,'FieldName',TFieldBoxProperty);
end;




end.
